// Loic Garrie F4HBG's RTTY baudot transmiter v1.0
// Inspired from Project Hollands Hoogte
// Transmit a string in baudot on pins 4 (mark) & 5 (space)

#include <stdint.h>
 
#define ARRAY_LEN 32
#define LETTERS_SHIFT 31
#define FIGURES_SHIFT 27
#define LINEFEED 2
#define CARRRTN  8
 
#define is_lowercase(ch)    ((ch) >= 'a' && (ch) <= 'z')
#define is_uppercase(ch)    ((ch) >= 'A' && (ch) <= 'Z')
 
unsigned long time;
 
char letters_arr[33] = "\000E\nA SIU\rDRJNFCKTZLWHYPQOBG\000MXV\000";
char figures_arr[33] = "\0003\n- \a87\r$4',!:(5\")2#6019?&\000./;\000";
 
void setup(){
  Serial.begin(9600);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);  
}
 
void loop(){ 
 time=millis();
 rtty_txstring("CQ CQ CQ DE F4HBG RTTY BAUDOT BEACON\r\n");
 time=millis()-time;
 Serial.println(time);
 delay(200); // Need to have a little break beetween strings
}
 
uint8_t char_to_baudot(char c, char *array)
{
  int i;
  for (i = 0; i < ARRAY_LEN; i++)
  {
    if (array[i] == c)
      return i;
  }
 
  return 0;
}
 
void rtty_txbyte(uint8_t b)
{
  int8_t i;
 
  rtty_txbit(0);
 
  for (i = 0; i < 5; i++)
  {
    if (b & (1 << i))
      rtty_txbit(1);
    else
      rtty_txbit(0);
  }
 
  rtty_txbit(1);
}
 
enum baudot_mode
{
  NONE,
  LETTERS,
  FIGURES
};
 
void rtty_txstring(char *str)
{
  enum baudot_mode current_mode = NONE;
  char c;
  uint8_t b;
 
  while (*str != '\0')
  {
    c = *str;
    /* Some characters are available in both sets */
    if (c == '\n')
    {
      rtty_txbyte(LINEFEED);
    }
    else if (c == '\r')
    {
      rtty_txbyte(CARRRTN);
    }
    else if (is_lowercase(*str) || is_uppercase(*str))
    {
      if (is_lowercase(*str))
      {
        c -= 32;
      }
 
      if (current_mode != LETTERS)
      {
        rtty_txbyte(LETTERS_SHIFT);
        current_mode = LETTERS;
      }
 
      rtty_txbyte(char_to_baudot(c, letters_arr));
    }
    else
    {
      b = char_to_baudot(c, figures_arr);
 
      if (b != 0 && current_mode != FIGURES)
      {
        rtty_txbyte(FIGURES_SHIFT);
        current_mode = FIGURES;
      }
 
      rtty_txbyte(b);
    }
 
    str++;
  }
}
 
// Bit transmit
void rtty_txbit (int bit) {
  if (bit) {
    // High - mark
    digitalWrite(4, HIGH);
    digitalWrite(5, LOW);
  } 
  else {
    // Low - space
    digitalWrite(5, HIGH);
    digitalWrite(4, LOW);
  }
  // Delay tuned to 50 baud. For some reason you can't do 22000 it just doesn't work.
  delayMicroseconds(10000);
  delayMicroseconds(12000);
}
